import zmq
from functools import partial
from zmq.eventloop.zmqstream import ZMQStream

from tfw.networking.serialization import serialize_all
from tfw.config import tfwenv
from tfw.networking.zmq_connector_base import ZMQConnectorBase


class ServerDownlinkConnector(ZMQConnectorBase):
    def __init__(self, zmq_context=None):
        super(ServerDownlinkConnector, self).__init__(zmq_context)
        self._zmq_sub_socket = self._zmq_context.socket(zmq.SUB)
        self._zmq_sub_socket.connect('tcp://localhost:{}'.format(tfwenv.PUBLISHER_PORT))
        self._zmq_sub_stream = ZMQStream(self._zmq_sub_socket)

        self.subscribe = partial(self._zmq_sub_socket.setsockopt_string, zmq.SUBSCRIBE)
        self.unsubscribe = partial(self._zmq_sub_socket.setsockopt_string, zmq.UNSUBSCRIBE)
        self.register_callback = self._zmq_sub_stream.on_recv


class ServerUplinkConnector(ZMQConnectorBase):
    def __init__(self, zmq_context=None):
        super(ServerUplinkConnector, self).__init__(zmq_context)
        self._zmq_push_socket = self._zmq_context.socket(zmq.PUSH)
        self._zmq_push_socket.connect('tcp://localhost:{}'.format(tfwenv.RECEIVER_PORT))

    def send(self, key, response):
        response['key'] = key
        self._zmq_push_socket.send_multipart(serialize_all(key, response))


class ServerConnector(ServerUplinkConnector, ServerDownlinkConnector):
    def __init__(self, zmq_context=None):
        super(ServerConnector, self).__init__(zmq_context)
